"""
Path     addons/source-python/pretty_status_list/pretty_status_list.py
Name     Pretty Status List
Version  1.00.002
Revision $Rev: 8 $
Author   (C) [#OMEGA] - K2


IMPORTANT

This plugin requires the following libraries:
k2tools
pygeoip


INFO

This is prettier status list.


AGREEMENT
 
By using this file you agree to the following terms:

This file is delivered 'as-is'. 
I will not take responsibility for any damage resulting of the use or not-use of this file. 

As for sharing permissions:
http://creativecommons.org/licenses/by-nc-nd/3.0/

If wish to alter the file/contents or reuse some parts please contact me.
"""

# =============================================================================
# Imports
# =============================================================================

# Source Python
from player_c import CPlayerGenerator

from core import GameEngine
from cvars import ServerVar
from entities.helpers import pointer_from_edict
from translations.strings import LangStrings

from core import GAME_NAME
from paths import ADDON_PATH
from paths import CFG_PATH
from paths import GAME_PATH
from paths import LOG_PATH

# Custom Source Python
import k2tools.config
from k2tools import pubcvarlib
from k2tools import smauth
from k2tools.cmdlib import AdvancedCommandManager
from k2tools.data import get_current_map
from k2tools.data import server_ip
from k2tools.data import server_port
from k2tools.misc import console_message
from k2tools.string import rfill
from k2tools.string import double_fill
from k2tools.threadlib import tick_delay

# Python
from sys import platform

# Custom Python
import pygeoip

# =============================================================================
# Addon Info
# =============================================================================

__version__ = '1.00.002 $Rev: 8 $'
__author__ = '[#OMEGA] - K2'

# =============================================================================
# Globals
# =============================================================================

lang = LangStrings('pretty_status_list')

mapname = 'unknown'

game = GAME_NAME
if game not in ['cstrike','csgo','dod','tf','empires']:
    game = 'unsupported'

# May need to be adjusted for new translations
INDEX = 6
USERID = 11
STEAMID = 13
IP = 15
COUNTRY = 8
TEAMINFO = 70
    
# =============================================================================
# Load & Unload 
# =============================================================================

def load():
    # Load Config File
    global config
    config = k2tools.config.load(CFG_PATH.joinpath('pretty_status_list.ini'), ADDON_PATH.joinpath('pretty_status_list','pretty_status_list-configspec.ini'), LOG_PATH.joinpath('pretty_status_list-config-errors.log'))
    
    # Load GeoIP File
    geoipfile = config['General']['GeoIPFile'] if config['General']['GeoIPFileIsAbsolute'] else GAME_PATH.joinpath(config['General']['GeoIPFile'])
    
    global geoip
    geoip = pygeoip.GeoIP(geoipfile, getattr(pygeoip, config['General']['GeoIPCacheMode']))
    
    # Register Commands
    kwargs = {
        'command': 'pstatus',
        'callback': (cmd_pstatus, ),
        'description': 'Shows information about the players',
        'console': True,
        'server': True,
    }
    
    if config['Auth']['Type'] == 'spauth':
        kwargs['auth'] = 1
        kwargs['auth_kwparams'] = []
        for item in ('Level', 'Permission', 'Flag'):
            if config['Auth']['SPAuth'][item]:
                kwargs['auth_kwparams'][item.lower()] = config['Auth']['SPAuth'][item]
    elif config['Auth']['Type'] == 'smauth':
        kwargs['pre_hook'] = (check_access, )
    
    AdvancedCommandManager.register(**kwargs)
    # Set Pub vars
    pubcvarlib.register('pstatus_version', __version__, tags=['pstatus', ])
    
def unload():
    # Unregister Commands
    AdvancedCommandManager.unregister('pstatus')
    # Cleanup pub vars
    pubcvarlib.unregister('pstatus_version')

# =============================================================================
# Functions
# =============================================================================

def check_access(av):
    if not check_sm_access(av):
        cmdlib._access_denied(av)
        return False
    return True

def check_sm_access(av):
    if not av['playerinfo']:
        return True
    p = smauth.PlayerAccess(player=av['playerinfo'])
    if config['Auth']['SMAuth']['Flag'] and not p.has_flag(config['Auth']['SMAuth']['Flag']):
        return False
    
    if config['Auth']['SMAuth']['Group'] and not p.in_group(config['Auth']['SMAuth']['Group']):
        return False
    
    return True

def cmd_pstatus(av):
    player = av['playerinfo']
    if player:
        cmduserid = player.get_userid()
        edict = player.get_edict()
        language = GameEngine.get_client_convar_value(edict.get_index(), 'cl_language')
    else:
        cmduserid = 0
        edict = None
        language = None # This will use server default
    # Parse player info
    team_msgs = [[double_fill(lang['team0'].get_string(language), TEAMINFO, '-')],
    [double_fill(lang['team1'].get_string(language), TEAMINFO, '-')],
    [double_fill(lang['%s_team2' % game].get_string(language), TEAMINFO, '-')],
    [double_fill(lang['%s_team3' % game].get_string(language), TEAMINFO, '-')]]
    for player in CPlayerGenerator():
        userid = player.get_userid()
        index = player.get_edict().get_index()
        msg = ''
        msg += rfill(str(index), INDEX)
        msg += rfill(str(userid), USERID)
        steamid = player.get_networkid_string()
        if steamid.startswith('STEAM_'):
            steamid = steamid[8:]
        msg += rfill(steamid, STEAMID)
        
        if not player.is_fake_client():
            inet = GameEngine.get_player_net_info(index)
            # Crashes with bots
            ip = inet.get_address().split(':')[0]
            msg += rfill(ip, IP)
            msg += rfill(geoip.country_code_by_addr(ip), COUNTRY)
        else:
            msg += rfill('BOT', IP)
            msg += rfill('BOT', COUNTRY)
        name = player.get_name()
        if isinstance(name, bytes):
            name = name.decode().encode('utf-8')
        msg += name
        msg += '\n'
        team_msgs[player.get_team_index()] += [msg,]
    # Print it to client console.
    if config['General']['PrintServerInfo']:
        con_print(edict, lang['serverinfo'].get_string(language) + '\n')
        con_print(edict, '\n')
        con_print(edict, 'IP/PORT     : ' + server_ip + ':' + server_port + '\n')
        con_print(edict, rfill(lang['hostname'].get_string(language),12) + ': ' + ServerVar('hostname').get_string() + '\n')
        con_print(edict, rfill(lang['map'].get_string(language),12) + ': ' + get_current_map() + '\n')
        if cmduserid:
            vec = edict.get_prop('m_vecOrigin').get_vector()
            pos = '%s, %s, %s' % (vec.x, vec.y, vec.z)
        else:
            pos = '0, 0, 0'
        con_print(edict, rfill(lang['position'].get_string(language),12) + ': ' + pos + '\n')
    con_print(edict, '\n')
    con_print(edict, lang['playerlist'].get_string(language) + '\n')
    con_print(edict, '\n')
    delay = 1
    for msgs in team_msgs:
        # Crashes ???? Still might be nececessary for ob, etc?
        #if not cmduserid:
        #    tick_delay(send_data, delay, cmduserid, msgs, edict, language)
        #    delay += 1
        #else:
        send_data(cmduserid, msgs, edict, language)

def send_data(cmduserid, msgs, edict, language):
    infoline = '%s%s%s%s%s%s\n' % (
        rfill(lang['index'].get_string(language), INDEX), 
        rfill(lang['userid'].get_string(language), USERID), 
        rfill(lang['steamid'].get_string(language), STEAMID), 
        rfill(lang['ip'].get_string(language), IP), 
        rfill(lang['country'].get_string(language), COUNTRY), 
        lang['name'].get_string(language))
    con_print(edict, msgs[0] + '\n')
    con_print(edict, infoline)
    con_print(edict, '\n')
    for msg in msgs[1:]:
        con_print(edict, msg)
    con_print(edict, '\n')

def con_print(edict, msg):
    if edict:
        GameEngine.client_printf(edict, msg)
    else:
        # Avoid issues with characters windows can't print
        if platform == 'win32':
            msg = msg.encode('cp850', errors='replace').decode()
        console_message(msg.replace('\n',''))